ARQ=~/bin/apache-jena-3.1.0/bin/arq
timestamp=`date '+%H:%M:%S'`
echo "# start time: $timestamp" > cityData.ttl

# Get the qnames for the geotagged entities within the city
curl --data-urlencode "query@getCityEntities.rq" https://query.wikidata.org/sparql -H "Accept: text/turtle" > cityEntities.ttl

# create queries that each pull data about subsets of the cityEntities.ttl list. They will have names like
# tempQueryFile1.rq, tempQueryFile2.rq, etc.; this also creates a sh file that calls them all.
$ARQ -query listSubjects.rq -data cityEntities.ttl --results=TSV | ./makePart2Queries.pl > callTempQueries.sh

# Do the queries that got created in the last step. 
./callTempQueries.sh > tempCityData.ttl

# For all the (instance is-a class) triples, list the classes, turn list into a query for superclasses. 
$ARQ -query listInstanceClasses.rq -data tempCityData.ttl  --results=TSV | ./makeGetClassesQuery.pl > getClassesQuery.rq

# Find the http://www.wikidata.org/entity/ objects that 
# don't have an and create a query to get their label values
$ARQ -query objectsThatNeedLabel.rq -data tempCityData.ttl  --results=TSV | ./makeGetObjectLabels.pl > getObjectLabelsQuery.rq

# Execute that query.
curl --data-urlencode "query@getObjectLabelsQuery.rq" https://query.wikidata.org/sparql -H "Accept: text/turtle" >> tempCityData.ttl

# Add data about superclasses to original tempCityData.ttl file. 
echo "# class information" >> tempCityData.ttl
curl --data-urlencode "query@getClassesQuery.rq" https://query.wikidata.org/sparql -H "Accept: text/turtle" >> tempCityData.ttl

# Normalize the triples (group them by subject) with a "select all" query
$ARQ -query selectAll.rq -data tempCityData.ttl >> cityData.ttl

rm tempQueryFile*rq

timestamp=`date '+%H:%M:%S'`
echo "# finish time: $timestamp" >> cityData.ttl
