#!/usr/bin/perl
use strict; 

# Make the following number smaller if curl requests for data are 
# timing out or too close to 59 seconds. maxLines = 80 made Chicago time out. 
my $maxLines = 50;          # maximum number of lines in each output file

# See "lang(" for how to control language of retrieved labels. 

# Two long variables that get defined before main body starts. 
my $queryHeader ;
my $queryFooter ; 

my $curlQuery = "";

$queryHeader =  "PREFIX p: <http://www.wikidata.org/prop/> 
PREFIX wgs84: <http://www.w3.org/2003/01/geo/wgs84_pos#>
PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#> 
PREFIX skos: <http://www.w3.org/2004/02/skos/core#> 

CONSTRUCT
{ ?s ?p ?o. 
  ?s ?p1 ?o1 . 
  ?s wgs84:lat ?lat . 
  ?s wgs84:long ?long .
  ?p rdfs:label ?pname .
  ?s wdt:P31 ?class .   
}
WHERE {
  VALUES ?s {
";

$queryFooter = "}
  # wdt:P131 means 'located in the administrative territorial entity' .
  ?s wdt:P131+ ?geoEntityWikidataID .  
      ?s p:P625 ?statement . # coordinate-location statement
  ?statement psv:P625 ?coordinate_node .
  ?coordinate_node wikibase:geoLatitude ?lat .
  ?coordinate_node wikibase:geoLongitude ?long .

  # Reduce the indirection used by Wikidata triples. Based on Tommy Potter query
  # at http://www.snee.com/bobdc.blog/2017/04/the-wikidata-data-model-and-yo.html.
  ?s ?directClaimP ?o .                   # Get the truthy triples. 
  ?p wikibase:directClaim ?directClaimP . # Find the wikibase properties linked
  ?p rdfs:label ?pname .                  # to the truthy triples' predicates.

  # the following VALUES clause is actually faster than just
  # having specific triple patterns for those 2 p1 values.
  ?s ?p1 ?o1 .
  VALUES ?p1 {
    schema:description
    rdfs:label        
    skos:altLabel
  }

  ?s wdt:P31 ?class . # Class membership. Pull this and higher level classes out in later query.
  
  # If only English names desired
  FILTER (isURI(?o1) || lang(?o1) = 'en' )
  # English + something else, follow this pattern: 
  # FILTER (isURI(?o1) || lang(?o1) = 'en' || lang(?o1) = 'ko')

# There are triple objects where it would be nice to have the labels, but it
# increases the execution time and chance of timeout too much. 
#  OPTIONAL { ?o1 rdfs:label ?o1label }

  FILTER(lang(?pname) = 'en')
}";

#### main body #####

#counter variables
my $fileNum = 1;
my $outputLineNum = 0;

my $outputFilename = "tempQueryFile" . $fileNum . ".rq";
open(OUTPUT,">$outputFilename");
print OUTPUT $queryHeader;

while(<>) {
if (!(/\?s/)) {  # if it's not the input header line
    $curlQuery =  "curl --data-urlencode \"query\@$outputFilename\" https://query.wikidata.org/sparql -H \"Accept: text/turtle\"\n";
    if ($outputLineNum > $maxLines) {

        # Finish the current file,
        print OUTPUT $queryFooter;
        $outputLineNum = 0; 
        close(OUTPUT);

        # add a line about the finished one to the script that will use it,
        print $curlQuery;
        # and start a new output file. 
        $fileNum++;
        $outputFilename = "tempQueryFile" . $fileNum . ".rq";
        open(OUTPUT,">$outputFilename");   
        print OUTPUT $queryHeader;
    }
    print OUTPUT $_;
    $outputLineNum++;
}
}

# Finish up the last one that was open. 
print OUTPUT $queryFooter;
close(OUTPUT);
        print $curlQuery;

